function MakeAuxiliaryFigures1_2()
%% Make Auxiliary Figures 1-2.
% Function makes one figure per cell line. Each figure includes single tumour
% fittings color-coded by treatment (5FU + Oxali, TL32711, 5FU + Oxali + TL32711, Control).
% Figures:
% - Auxiliary Figure 1: HCT116 tumour xenografts;
% - Auxiliary Figure 2: LoVo tumour xenografts.

% Read fitted tumour growth in mice xenografts, as generated by FitInVivoTumourVolume
data = load('in_vivo_tumour_volume_fittings.mat');

% Make supplementary figures
MakePerCellLineXenograftFigure(data.exp_fit_hct, 1500, 'HCT116');
MakePerCellLineXenograftFigure(data.exp_fit_lovo, 1000, 'LoVo');
end

function MakePerCellLineXenograftFigure(t, ylim2, cell_line)
%% Make figure for a cell line xenograft.
% Function plots 4 rows of panels, one row for each treatment (5FU + Oxali,
% TL32711, 5FU + Oxali + TL32711, Control). Each panel includes the measured
% tumour volume (square symbols) and the corresponding fitted tumour growth
% (solid line) for one tumour.
% Input arguments:
% - t: table with tumour volume data and fitted parameters, as returned by 
%   RunCellLineFittings in FitInVivoTumourVolume.m. One row per tumour/days;
% - ylim2: upper limit for y axis limits;
% - cell_line: string, name of cell line (HCT116 or LoVo).

figure();
matlab_utilities.MaximizeFigure();
p = matlab_utilities.panel.panel();

treatments = categories(t.treatment);
% Have as many rows as the maximum number of mice present per treatment
stats = grpstats(t, {'cell_line', 'treatment'}, {}, 'DataVars', false);
number_of_rows = max(stats.GroupCount ./ numel(categories(categorical(t.days))));

days = 0 : 1 : max(t.days) + 1;

p.pack('v', {0.01, 0.99});
p(2).pack('v', numel(treatments));

colors = {'g', 'b', 'r', 'k'}; % (5FU + Oxali, TL32711, 5FU + Oxali + TL32711, Control)

for i = 1:numel(treatments)
    treatment_table = t(t.treatment == treatments{i}, :);
    tumours = categories(removecats(treatment_table.tumour_id));
    p(2, i).pack('h', number_of_rows); % one row of panels for each treatment
    tp = title(p(2, i), sprintf('%s', treatments{i}));
    set(tp, 'color', colors{i});
    for tumour = 1 : numel(tumours)
        p(2, i, tumour).select(); % One panel for each tumour
        tumour_table = treatment_table(...
            categorical(treatment_table.tumour_id) == tumours{tumour}, :);
        % Plot experimental measurements
        plot(tumour_table.days, tumour_table.tumour_volume, 's',...
            'MarkerSize', 4, 'MarkerFaceColor', colors{i}, 'MarkerEdgeColor', colors{i},...
            'displayname', sprintf('%s - exp', treatments{i}));
        hold on;
        % Compute fitted tumour growth and plot
        v = TumourVolume(days, tumour_table.Kd(1), tumour_table.V0(1),...
            tumour_table.a(1), tumour_table.K(1), tumour_table.treatment_start_day(1));
        plot(days, v, '-', 'color', colors{i}, 'linewidth', 1,...
            'displayname', sprintf('%s - fit', treatments{i}));
        
        set(gca, 'Xlim', [-1, max(days)],...
            'Ylim', [0 ylim2], 'XTick', 0 : 7 : max(days),...
            'box', 'on', 'linewidth', 1, 'xcolor', 'k', 'ycolor', 'k');
        %Hide XTickLabels and YTickLabels except for the last row/first
        %column
        if i < numel(treatments)
            set(gca, 'XTickLabel', {});
        end
        if tumour > 1
            set(gca, 'YTickLabel', {});
        end 
    end
end

hx = xlabel(p(2), 'Time [day]');
hy = ylabel(p(2), 'Tumour volume [mm^3]');
ht = title(p, sprintf('%s xenografts', cell_line));
set([hx, hy, ht], 'color', 'k');

p.fontname = 'arial';
p.fontsize = 8;

p.descendants.margin = [3 8 3 5];
p(1).marginbottom = 0;
p.margin = [13 10 4 5];

p.export(fullfile('figures', sprintf('AuxiliaryFigure%d.pdf',...
    matlab_utilities.iif(strcmp(cell_line, 'HCT116'), 1, 2))),...
    '-rx', '-w267', '-h180');
end
